@@ -6,6 +6,7 @@ from abc import ABC, abstractmethod
 from urllib.parse import urlparse
 
 import boto3
+import typing
 
 
 class LandingZone(ABC):
@@ -13,8 +14,22 @@ class LandingZone(ABC):
     def upload(self, src_path: str, dst_path: str, suffix: str = "", prefix: str = None):
         pass
 
+    @abstractmethod
+    def list_files(self, bucket: str, prefix: str) -> typing.List[str]:
+        pass
+
+    @abstractmethod
+    def download_file_into(self, bucket: str, prefix: str, f) -> typing.List[str]:
+        pass
+
 
 class FilesystemLandingZone(LandingZone):
+    def list_files(self, bucket: str, prefix: str) -> typing.List[str]:
+        pass
+
+    def download_file_into(self, bucket: str, prefix: str, f) -> typing.List[str]:
+        pass
+
     def upload(
         self,
         src_path: str,
@@ -24,11 +39,14 @@ class FilesystemLandingZone(LandingZone):
     ):
         u = urlparse(dst_path)
         with open(src_path, "rb") as f_in:
+
+            os.makedirs(u.path, exist_ok=True)
+
             with tempfile.NamedTemporaryFile(
                 "wb",
                 prefix=os.path.basename(src_path),
                 delete=False,
-                dir="." + u.path,
+                dir=u.path,
                 suffix=suffix,
             ) as f:
                 with gzip.open(f, "wb") as f_out:
@@ -39,6 +57,20 @@ class AmazonS3LandingZone(LandingZone):
     def __init__(self):
         self.client = boto3.client("s3")
 
+    def list_files(self, bucket: str, prefix: str) -> typing.List[str]:
+        kwargs = {'Bucket': bucket, 'Prefix': prefix}
+        while True:
+            resp = self.client.list_objects_v2(**kwargs)
+            for obj in resp['Contents']:
+                yield obj['Key']
+            try:
+                kwargs['ContinuationToken'] = resp['NextContinuationToken']
+            except KeyError:
+                break
+
+    def download_file_into(self, bucket: str, key: str, f) -> typing.List[str]:
+        return self.client.download_fileobj(bucket, key, f)
+
     def upload(self, src_path: str, dst_path: str, suffix: str = "", prefix: str = None):
         u = urlparse(dst_path)
 
