@@ -272,7 +272,7 @@ def map_attribute_type(t: str, col):
     elif t == "numeric":
         scale = col.get("scale", 4)
         precision = col.get("precision", 15)
-        if len(scale) > 0 and len(precision) > 0:
+        if scale and precision and scale > 0 and precision > 0:
             return DECIMAL(scale=scale, precision=precision)
         else:
             return DECIMAL()
